@@ -1,9 +1,11 @@
 package cmd
 
 import (
+	"bytes"
 	"fmt"
 	"os"
 	"syscall"
+	"text/template"
 
 	"github.com/spf13/cobra"
 	"github.com/spf13/pflag"
@@ -72,6 +74,17 @@ func printToStdout(gs *state.GlobalState, s string) {
 	}
 }
 
+func getExampleText(gs *state.GlobalState, tpl string) string {
+	var exampleText bytes.Buffer
+	exampleTemplate := template.Must(template.New("").Parse(tpl))
+
+	if err := exampleTemplate.Execute(&exampleText, gs.BinaryName); err != nil {
+		gs.Logger.WithError(err).Error("Error during help example generation")
+	}
+
+	return exampleText.String()
+}
+
 // Trap Interrupts, SIGINTs and SIGTERMs and call the given.
 func handleTestAbortSignals(gs *state.GlobalState, gracefulStopHandler, onHardStop func(os.Signal)) (stop func()) {
 	gs.Logger.Debug("Trapping interrupt signals so k6 can handle them gracefully...")
