@@ -34,10 +34,11 @@ const defaultConfigFileName = "config.json"
 type GlobalState struct {
 	Ctx context.Context
 
-	FS      afero.Fs
-	Getwd   func() (string, error)
-	CmdArgs []string
-	Env     map[string]string
+	FS         afero.Fs
+	Getwd      func() (string, error)
+	BinaryName string
+	CmdArgs    []string
+	Env        map[string]string
 
 	DefaultFlags, Flags GlobalFlags
 
@@ -58,6 +59,8 @@ type GlobalState struct {
 // global variables and functions from the os package. Anywhere else, things
 // like os.Stdout, os.Stderr, os.Stdin, os.Getenv(), etc. should be removed and
 // the respective properties of globalState used instead.
+//
+//nolint:forbidigo
 func NewGlobalState(ctx context.Context) *GlobalState {
 	isDumbTerm := os.Getenv("TERM") == "dumb"
 	stdoutTTY := !isDumbTerm && (isatty.IsTerminal(os.Stdout.Fd()) || isatty.IsCygwinTerminal(os.Stdout.Fd()))
@@ -93,12 +96,18 @@ func NewGlobalState(ctx context.Context) *GlobalState {
 		confDir = ".config"
 	}
 
+	binary, err := os.Executable()
+	if err != nil {
+		binary = "k6"
+	}
+
 	defaultFlags := GetDefaultFlags(confDir)
 
 	return &GlobalState{
 		Ctx:          ctx,
 		FS:           afero.NewOsFs(),
 		Getwd:        os.Getwd,
+		BinaryName:   filepath.Base(binary),
 		CmdArgs:      os.Args,
 		Env:          env,
 		DefaultFlags: defaultFlags,
