@@ -1,5 +1,6 @@
 // Heavily influenced by the fantastic work by @dop251 for https://github.com/dop251/goja
-
+//
+//nolint:forbidigo
 package tc39
 
 import (
@@ -7,7 +8,7 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"os"
 	"path"
 	"runtime"
@@ -47,7 +48,6 @@ var (
 	featuresBlockList = []string{
 		"BigInt",                      // not supported at all
 		"IsHTMLDDA",                   // not supported at all
-		"generators",                  // not supported in a meaningful way IMO
 		"async-iteration",             // not supported at all
 		"top-level-await",             // not supported at all
 		"String.prototype.replaceAll", // not supported at all, Stage 4 since 2020
@@ -80,10 +80,8 @@ var (
 
 		"array-find-from-last", // stage 3 as of 2021 https://github.com/tc39/proposal-array-find-from-last
 	}
-	skipWords = []string{"yield", "generator", "Generator", "module"}
+	skipWords = []string{"module"}
 	skipList  = map[string]bool{
-		"test/built-ins/Object/seal/seal-generatorfunction.js": true,
-
 		"test/built-ins/Date/parse/without-utc-offset.js": true, // some other reason ?!? depending on local time
 
 		"test/built-ins/Array/prototype/concat/arg-length-exceeding-integer-limit.js": true, // takes forever and is broken
@@ -91,6 +89,10 @@ var (
 		"test/built-ins/Array/prototype/unshift/clamps-to-integer-limit.js":           true, // takes forever and is broken
 		"test/built-ins/Array/prototype/unshift/throws-if-integer-limit-exceeded.js":  true, // takes forever and is broken
 
+		// async generator
+		"test/built-ins/Object/seal/seal-asyncgeneratorfunction.js": true,
+		"test/language/expressions/async-generator/name.js":         true,
+
 		// wrong tests after change in the specification https://github.com/tc39/test262/issues/3407
 		"test/language/expressions/prefix-decrement/S11.4.5_A6_T3.js":        true,
 		"test/language/expressions/prefix-decrement/S11.4.5_A6_T2.js":        true,
@@ -159,36 +161,53 @@ var (
 		"test/annexB/built-ins/RegExp/legacy-accessors/",
 		"test/language/literals/string/legacy-", // legecy string escapes
 
-		// Async/Promise and other totally unsupported functionality
+		"test/built-ins/Atomics",
+		"test/built-ins/BigInt",
+		"test/built-ins/SharedArrayBuffer",
+
+		// Async generators are not currently supported
 		"test/built-ins/AsyncFromSyncIteratorPrototype",
 		"test/built-ins/AsyncGeneratorFunction",
 		"test/built-ins/AsyncGeneratorPrototype",
 		"test/built-ins/AsyncIteratorPrototype",
-		"test/built-ins/Atomics",
-		"test/built-ins/BigInt",
-		"test/built-ins/SharedArrayBuffer",
-		"test/language/expressions/dynamic-import",
-		"test/language/module-code/top-level-await",
-
-		// generator methods seem to just not work at all
-		"test/language/arguments-object/cls-decl-gen-met",
-		"test/language/arguments-object/cls-decl-private-gen-meth",
-		"test/language/arguments-object/cls-expr-gen-meth",
-		"test/language/arguments-object/cls-expr-private-gen-meth",
-		"test/language/eval-code/direct/gen-",
-		"test/language/eval-code/direct/async-gen-",
-		"test/language/expressions/class/gen-method",
-		"test/language/expressions/class/dstr/gen-meth",
-		"test/language/expressions/class/dstr/private-gen-meth",
-		"test/language/expressions/object/dstr/gen-meth",
-		"test/language/statements/class/gen-method",
-		"test/language/statements/class/dstr/gen-meth",
-		"test/language/statements/class/dstr/private-gen-meth",
-
-		"test/built-ins/Function/prototype/toString/async",
-		"test/built-ins/Function/prototype/toString/async",
-		"test/built-ins/Function/prototype/toString/generator",
-		"test/built-ins/Function/prototype/toString/proxy-async",
+		"test/language/eval-code/direct/async-gen",
+		"test/language/expressions/async-generator",
+		"test/language/expressions/class/elements/after-same-line-gen-rs-static-async-generator",
+		"test/language/expressions/class/elements/after-same-line-method-rs-static-async-generator",
+		"test/language/expressions/class/elements/after-same-line-static-async-method-rs-static-async-generator",
+		"test/language/expressions/class/elements/after-same-line-static-gen-rs-static-async-generator",
+		"test/language/expressions/class/elements/after-same-line-static-method-rs-static-async-generator",
+		"test/language/expressions/class/elements/multiple-definitions-rs-static-async-generator",
+		"test/language/expressions/class/elements/multiple-stacked-definitions-rs-static-async-generator",
+		"test/language/expressions/class/elements/new-no-sc-line-gen-rs-static-async-generator",
+		"test/language/expressions/class/elements/new-no-sc-line-method-rs-static-async-generator",
+		"test/language/expressions/class/elements/new-sc-line-gen-rs-static-async-generator",
+		"test/language/expressions/class/elements/new-sc-line-method-rs-static-async-generator",
+		"test/language/expressions/class/elements/private-async-generator",
+		"test/language/expressions/class/elements/private-static-async-generator",
+		"test/language/expressions/class/elements/regular-definitions-rs-static-async-generator",
+		"test/language/expressions/class/elements/same-line-async-method-rs-static-async-generator",
+		"test/language/expressions/class/elements/same-line-gen-rs-static-async-generator",
+		"test/language/expressions/class/elements/same-line-method-rs-static-async-generator",
+		"test/language/expressions/class/elements/wrapped-in-sc-rs-static-async-generator",
+		"test/language/statements/class/elements/after-same-line-gen-rs-static-async-generator",
+		"test/language/statements/class/elements/after-same-line-method-rs-static-async-generator",
+		"test/language/statements/class/elements/after-same-line-static-async-method-rs-static-async-generator",
+		"test/language/statements/class/elements/after-same-line-static-gen-rs-static-async-generator",
+		"test/language/statements/class/elements/after-same-line-static-method-rs-static-async-generator",
+		"test/language/statements/class/elements/multiple-definitions-rs-static-async-generator",
+		"test/language/statements/class/elements/multiple-stacked-definitions-rs-static-async-generator",
+		"test/language/statements/class/elements/new-no-sc-line-method-rs-static-async-generator",
+		"test/language/statements/class/elements/new-sc-line-gen-rs-static-async-generator",
+		"test/language/statements/class/elements/new-sc-line-method-rs-static-async-generator",
+		"test/language/statements/class/elements/private-async-generator",
+		"test/language/statements/class/elements/private-static-async-generator",
+		"test/language/statements/class/elements/regular-definitions-rs-static-async-generator",
+		"test/language/statements/class/elements/same-line-async-method-rs-static-async-generator",
+		"test/language/statements/class/elements/same-line-gen-rs-static-async-generator",
+		"test/language/statements/class/elements/same-line-method-rs-static-async-generator",
+		"test/language/statements/class/elements/wrapped-in-sc-rs-static-async-generator",
+		"test/language/statements/switch/scope-lex-async-generator.js",
 
 		"test/built-ins/FinalizationRegistry", // still in proposal
 
@@ -198,7 +217,8 @@ var (
 		"test/built-ins/Object/prototype/__lookup", // AnnexB lookupGetter lookupSetter
 		"test/built-ins/Object/prototype/__define", // AnnexB defineGetter defineSetter
 
-		"test/language/module-code/", // this requires that we rewrite the js package in a way that it can this tests - which is unlikely to ever happen
+		"test/language/module-code/",               // this requires that we rewrite the js package in a way that it can this tests - which is unlikely to ever happen
+		"test/language/expressions/dynamic-import", // not supported
 	}
 )
 
@@ -261,7 +281,7 @@ func parseTC39File(name string) (*tc39Meta, string, error) {
 	}
 	defer f.Close() //nolint:errcheck,gosec
 
-	b, err := ioutil.ReadAll(f)
+	b, err := io.ReadAll(f)
 	if err != nil {
 		return nil, "", err
 	}
@@ -530,7 +550,7 @@ func (ctx *tc39TestCtx) init() {
 	ctx.prgCache = make(map[string]*goja.Program)
 	ctx.errors = make(map[string]string)
 
-	b, err := ioutil.ReadFile("./breaking_test_errors.json")
+	b, err := os.ReadFile("./breaking_test_errors.json")
 	if err != nil {
 		panic(err)
 	}
@@ -557,7 +577,7 @@ func (ctx *tc39TestCtx) compile(base, name string) (*goja.Program, error) {
 		}
 		defer f.Close() //nolint:gosec,errcheck
 
-		b, err := ioutil.ReadAll(f)
+		b, err := io.ReadAll(f)
 		if err != nil {
 			return nil, err
 		}
@@ -628,7 +648,7 @@ func (ctx *tc39TestCtx) runTC39Script(name, src string, includes []string, vm *g
 }
 
 func (ctx *tc39TestCtx) runTC39Tests(name string) {
-	files, err := ioutil.ReadDir(path.Join(ctx.base, name))
+	files, err := os.ReadDir(path.Join(ctx.base, name))
 	if err != nil {
 		ctx.t.Fatal(err)
 	}
