@@ -15,9 +15,7 @@ type TestPreInitState struct {
 	BuiltinMetrics *metrics.BuiltinMetrics
 	KeyLogger      io.Writer
 	LookupEnv      func(key string) (val string, ok bool)
-
-	// TODO: replace with logrus.FieldLogger when all of the tests can be fixed
-	Logger *logrus.Logger
+	Logger         logrus.FieldLogger
 }
 
 // TestRunState contains the pre-init state as well as all of the state and
